@@ -16,15 +16,18 @@
 #define EUC
 #undef SJIS
 
-#ifdef HAVE_A_OUT_H
 
-/* define USE_DLN to load object file(.o). */
-#define USE_DLN
-#ifdef USE_DLN
-#define LIBC_NAME "libc.a"
-#define DLN_DEFAULT_PATH "/lib:/usr/lib:."
+/* define USE_DL to load object file(.o). */
+#define USE_DL
+
+/* a.out.h or dlopen() needed to load object */
+#if !defined(HAVE_DLOPEN) || !defined(HAVE_A_OUT_H)
+# undef USE_DL
 #endif
 
+#ifdef USE_MY_DLN
+#  define LIBC_NAME "libc.a"
+#  define DLN_DEFAULT_PATH "/lib:/usr/lib:."
 #endif
 
 /* define USE_DBM to use dbm class. */
