@@ -7,7 +7,7 @@
   $Date: 1994/08/12 04:47:52 $
   created at: Thu Sep 30 14:18:32 JST 1993
 
-  Copyright (C) 1994 Yukihiro Matsumoto
+  Copyright (C) 1995 Yukihiro Matsumoto
 
 ************************************************/
 
@@ -18,10 +18,19 @@
 #include <stdio.h>
 
 #include "regex.h"
-typedef struct Regexp {
-        struct re_pattern_buffer pat;
-        struct re_registers regs;
-} Regexp;
+
+typedef struct re_pattern_buffer Regexp;
+
+struct match {
+    UINT len;
+    char *ptr;
+    struct re_registers regs;
+};
+
+extern struct match last_match;
+
+#define BEG(no) last_match.regs.beg[no]
+#define END(no) last_match.regs.end[no]
 
 VALUE re_regcomp();
 VALUE re_regsub();
