@@ -8,7 +8,7 @@
 ;;;
 
 (defconst ruby-block-beg-re
-  "class\\|module\\|def\\|if\\|unless\\|case\\|while\\|until\\|for\\|protect"
+  "class\\|module\\|def\\|if\\|case\\|while\\|for\\|begin"
   )
 
 (defconst ruby-block-mid-re
@@ -18,7 +18,7 @@
 (defconst ruby-block-end-re "end")
 
 (defconst ruby-delimiter
-  (concat "[$/<(){}#\"'`]\\|\\[\\|\\]\\|\\b\\("
+  (concat "[$/(){}#\"'`]\\|\\[\\|\\]\\|\\b\\("
 	  ruby-block-beg-re "\\|" ruby-block-end-re "\\)\\b")
   )
 
@@ -170,13 +170,15 @@ The variable ruby-indent-level controls the amount of indentation.
 	   ((or (string= "\"" w)	;skip string
 		(string= "'" w)
 		(string= "`" w))
-	    (if (search-forward w indent-point t)
-		nil
+	    (cond 
+	     ((string= w (char-to-string (char-after (point))))
+	      (forward-char 1))
+	     ((re-search-forward (format "[^\\]%s" w) indent-point t)
+		nil)
+	     (t
 	      (goto-char indent-point)
-	      (setq in-string t)))
-	   ((or (string= "/" w)
-		(string= "<" w))
-	    (if (string= "<" w) (setq w ">"))
+	      (setq in-string t))))
+	   ((string= "/" w)
 	    (let (c)
 	      (save-excursion
 		(goto-char pnt)
@@ -189,7 +191,9 @@ The variable ruby-indent-level controls the amount of indentation.
 		    (and (eq c ?w)
 			 (save-excursion
 			   (forward-word -1)
-			   (looking-at ruby-block-beg-re))))
+			   (or 
+			    (looking-at ruby-block-beg-re)
+			    (looking-at ruby-block-mid-re)))))
 		(if (search-forward w indent-point t)
 		    nil
 		  (goto-char indent-point)
@@ -257,6 +261,120 @@ The variable ruby-indent-level controls the amount of indentation.
 	    (error (format "bad string %s" w)))))))
     (list in-string in-paren (car nest) depth)))
 
+(defun ruby-parse-region (start end)
+  (let ((indent-point end)
+	(indent 0)
+	(in-string nil)
+	(in-paren nil)
+	(depth 0)
+	(nest nil))
+    (save-excursion
+      (if start
+	  (goto-char start)
+	(ruby-beginning-of-defun))
+      (while (and (> indent-point (point))
+		  (re-search-forward ruby-delimiter indent-point t))
+	(let ((w (buffer-substring (match-beginning 0) (match-end 0)))
+	      (pnt (match-beginning 0)))
+	  (cond
+	   ((or (string= "\"" w)	;skip string
+		(string= "'" w)
+		(string= "`" w))
+	    (cond 
+	     ((string= w (char-to-string (char-after (point))))
+	      (forward-char 1))
+	     ((re-search-forward (format "[^\\]%s" w) indent-point t)
+		nil)
+	     (t
+	      (goto-char indent-point)
+	      (setq in-string t))))
+	   ((or (string= "/" w)
+		(string= "<" w))
+	    (if (string= "<" w) (setq w ">"))
+	    (let (c)
+	      (save-excursion
+		(goto-char pnt)
+		(skip-chars-backward " \t")
+		(setq c (char-after (1- (point))))
+		(if c (setq c (char-syntax c))))
+	      (if (or (eq c ?.)
+		      (and (eq c ?w)
+			   (save-excursion
+			     (forward-word -1)
+			     (or 
+			      (looking-at ruby-block-beg-re)
+			      (looking-at ruby-block-mid-re)))))
+		  (cond
+		   ((string= w (char-to-string (char-after (point))))
+		    (forward-char 1))
+		   ((re-search-forward (format "[^\\]%s" w) indent-point t)
+		    nil)
+		   (t
+		    (goto-char indent-point)
+		    (setq in-string t))))))
+	   ((string= "$" w)		;skip $char
+	    (forward-char 1))
+	   ((string= "#" w)		;skip comment
+	    (forward-line 1))
+	   ((string= "(" w)		;skip to matching paren
+	    (let ((orig depth))
+	      (setq nest (cons (point) nest))
+	      (setq depth (1+ depth))
+	      (while (and (/= depth orig)
+			  (re-search-forward "[()]" indent-point t))
+		(cond
+		 ((= (char-after (match-beginning 0)) ?\( )
+		  (setq nest (cons (point) nest))
+		  (setq depth (1+ depth)))
+		 (t
+		  (setq nest (cdr nest))
+		  (setq depth (1- depth)))))
+	      (if (> depth orig) (setq in-paren ?\())))
+	   ((string= "[" w)		;skip to matching paren
+	    (let ((orig depth))
+	      (setq nest (cons (point) nest))
+	      (setq depth (1+ depth))
+	      (while (and (/= depth orig)
+			  (re-search-forward "\\[\\|\\]" indent-point t))
+		(cond
+		 ((= (char-after (match-beginning 0)) ?\[ )
+		  (setq nest (cons (point) nest))
+		  (setq depth (1+ depth)))
+		 (t
+		  (setq nest (cdr nest))
+		  (setq depth (1- depth)))))
+	      (if (> depth orig) (setq in-paren ?\[))))
+	   ((string= "{" w)		;skip to matching paren
+	    (let ((orig depth))
+	      (setq nest (cons (point) nest))
+	      (setq depth (1+ depth))
+	      (while (and (/= depth orig)
+			  (re-search-forward "[{}]" indent-point t))
+		(cond
+		 ((= (char-after (match-beginning 0)) ?{ )
+		  (setq nest (cons (point) nest))
+		  (setq depth (1+ depth)))
+		 (t
+		  (setq nest (cdr nest))
+		  (setq depth (1- depth)))))
+	      (if (> depth orig) (setq in-paren ?{))))
+	   ((string-match ruby-block-end-re w)
+	    (setq nest (cdr nest))
+	    (setq depth (1- depth)))
+	   ((string-match ruby-block-beg-re w)
+	    (let (c)
+	      (save-excursion
+		(goto-char pnt)
+		(skip-chars-backward " \t")
+		(setq c (char-after (1- (point)))))
+	      (if (or (null c) (= c ?\n) (= c ?\;))
+		  (progn
+		    (setq nest (cons (point) nest))
+		    (setq depth (1+ depth))))))
+	   (t
+	    (error (format "bad string %s" w)))))))
+    (list in-string in-paren (car nest) depth)))
+
 (defun ruby-calculate-indent (&optional parse-start)
   (save-excursion
     (beginning-of-line)
@@ -315,12 +433,12 @@ An end of a defun is found by moving forward from the beginning of one."
 
 (defun ruby-reindent-then-newline-and-indent ()
   (interactive "*")
-  (save-excursion
-    (delete-region (point) (progn (skip-chars-backward " \t") (point))))
   (insert ?\n)
   (save-excursion
     (forward-line -1)
-    (indent-according-to-mode))
+    (indent-according-to-mode)
+    (end-of-line)
+    (delete-region (point) (progn (skip-chars-backward " \t") (point))))
   (indent-according-to-mode))
 
 (defun ruby-encomment-region (beg end)
